@@ -7,7 +7,7 @@ Max line length:    80
 Programmer:         Volker Kuhlmann
 
 
-wav2cdr 2.1 Copyright (C) 1997, 1998 Volker Kuhlmann
+wav2cdr 2.3.2 Copyright (C) 1997, 1998, 1999 Volker Kuhlmann
 This program is free software under the terms of the GNU General Public License
 version 2 (or later, at your option).
 See the file COPYING for details about license terms and warranty.
@@ -68,12 +68,12 @@ static cmdarg_t
 	cmdarg = CMDARG_DEFAULT;	/* cmd args, set to default values */
 
 /* size of input data in bytes, not counting headers
-	size not available when == -1
-	Can not make this unsigned int because that may be only 16 bit.
+	size not available when == -1 (therefore must be a signed type)
+	Can not make this int because that may be only 16 bit.
 	Can not make this size_t because that is usually unsigned int.
 	Should be the same type as get_file_size().
 */
-static long input_size;
+static signed long input_size;
 
 
 
@@ -86,6 +86,7 @@ static UINT16 get_uint16_bytes (const char *s);
 static UINT16 get_uint16_blocks (const char *s);
 #endif
 static void check_monotonous_cuts (void);
+static signed long get_neg_cut (signed long cut);
 
 
 
@@ -107,17 +108,19 @@ enum {  /* all must be != 0, and != any of the short option letters */
 	o_in = 1, o_out, o_iscale, o_fscale, o_ssil, o_esil,
 	o_indflt, o_inwav, o_incdr, o_inraw,
 	o_todflt, o_toraw, o_tocdr, o_towav, 
-	o_swapwords0, o_swapwords1, o_silthresh, o_sildelay
+	o_swapwords0, o_swapwords1, o_silthresh, o_sildelay, o_fadein, o_fadeout
 	};
-const string shortoptions[] = "hIiOour:w:";
+const string shortoptions[] = "hIiOour:Vw:";
 const struct option longoptions[] = {
 	{"h",				no_argument, (int *) &cmdarg.usage, 'u'},
 	{"u",				no_argument, (int *) &cmdarg.usage, 'u'},
 	{"usage",			no_argument, (int *) &cmdarg.usage, 'u'},
 	{"help",			no_argument, (int *) &cmdarg.help, 'H'},
+	{"tocdr",			no_argument, NULL, o_tocdr},
 	{"towav",			no_argument, NULL, o_towav},
 	{"toraw",			no_argument, NULL, o_toraw},
 	{"inraw",			no_argument, NULL, o_inraw},
+	{"inwav",			no_argument, NULL, o_inwav},
 	{"incdr",			no_argument, NULL, o_incdr},
 	#if 0
 	{"ifmt",			required_argument, NULL, o_iformat},
@@ -151,7 +154,11 @@ const struct option longoptions[] = {
 	{"silenceinfo",	 	no_argument, (int *) &cmdarg.silenceinfo, TRUE},
 	{"silencethresh", 	required_argument, NULL, o_silthresh},
 	{"silencedelay", 	required_argument, NULL, o_sildelay},
+	{"fadein",		 	required_argument, NULL, o_fadein},
+	{"fadeout",		 	required_argument, NULL, o_fadeout},
 	{"quiet", 			no_argument, (int *) &cmdarg.quiet, TRUE},
+	{"version", 		no_argument, NULL, 'V'},
+	{"verbose", 		no_argument, (int *) &cmdarg.verbose, TRUE},
 	{NULL,				no_argument, NULL, 0}
 	};
 
@@ -177,6 +184,9 @@ const struct option longoptions[] = {
 		case 'u':
 			cmdarg.usage = TRUE;
 			break;
+		case 'V':
+			cmdarg.version = TRUE;
+			break;
 		case o_swapwords1:
 			cmdarg.swapwords = TRUE;
 			break;
@@ -187,10 +197,18 @@ const struct option longoptions[] = {
 			cmdarg.informat = AF_raw;
 			cmdarg.little_input = RAW_FORMAT_DEFAULT_IS_LITTLE;
 			break;
+		case o_inwav:
+			cmdarg.informat = AF_wav;
+			cmdarg.little_input = TRUE;
+			break;
 		case o_incdr:
 			cmdarg.informat = AF_cdr;
 			cmdarg.little_input = FALSE;
 			break;
+		case o_tocdr:
+			cmdarg.outformat = AF_cdr;
+			cmdarg.little_output = FALSE;
+			break;
 		case o_towav:
 			cmdarg.outformat = AF_wav;
 			cmdarg.little_output = TRUE;
@@ -254,6 +272,12 @@ const struct option longoptions[] = {
 		case o_sildelay:
 			cmdarg.silence_delay = get_num_unit (optarg) / CDAUDIOSECTORSIZE;
 			break;
+		case o_fadein:
+			cmdarg.fadein = get_num_unit (optarg);
+			break;
+		case o_fadeout:
+			cmdarg.fadeout = get_num_unit (optarg);
+			break;
 		case 0:
 		case -1:
 			break;
@@ -375,12 +399,14 @@ int hs;
 		fprintf (msgfile, "Scale factor is negative. "
 			"Will proceed and assume you know what you are doing.\n");
 
+	/* initialise input data size */
 	input_size = get_file_size (cmdarg.infilename, stdin);
 	/* Adjust the input data size to the amount of actual data only,
 		excluding any headers.  FIXME: this is dodgy... */
 	hs = get_af_header_size (cmdarg.informat);
 	if (input_size != -1)
 		input_size -= (input_size >= (signed long) hs) ? hs : input_size;
+
 	if (cmdarg.numcuts > 0) {
 		if ((cmdarg.numcuts > 2) AND (cmdarg.outfilename == NULL))
 			exit_error (ERR_CMDARG, 
@@ -398,6 +424,10 @@ int hs;
 		msgfile = stdout;
 		usage ();
 	}
+	if (cmdarg.version) {
+		msgfile = stdout;
+		version ();
+	}
 
 	if ((cmdarg.iscale != 100) AND (cmdarg.fscale != 1.0)) {
 		fprintf (msgfile, 
@@ -414,6 +444,10 @@ int hs;
 		exit_error (ERR_CMDARG,
 					"silence delay must be >= 1", "");
 
+	if (cmdarg.fadeout > 0 AND cmdarg.numcuts == 0 AND input_size == -1)
+		exit_error (ERR_CMDARG,
+					"can't fade out when input size can't be determined", "");
+	
 } /* check_cmd_args() */
 
 
@@ -457,8 +491,15 @@ const string
 const string *AF_nameidx[] = AF_NAMEIDX;
 cdseccount_t firstcut = 0, lastcut = 0;
 string insize[80], silence[80];
+string stasil[TIMESTRSIZE], endsil[TIMESTRSIZE];
+string filength[TIMESTRSIZE], folength[TIMESTRSIZE];
 string stime[TIMESTRSIZE], sfirstcut[TIMESTRSIZE], slastcut[TIMESTRSIZE];
 
+	timeprintf (stasil, cmdarg.startsilence);
+	timeprintf (endsil, cmdarg.endsilence);
+	timeprintf (filength, cmdarg.fadein);
+	timeprintf (folength, cmdarg.fadeout);
+
 	if (input_size == -1)
 		strcpy (insize, "not sizeable/seekable (reading pipe?)");
 	else {
@@ -471,6 +512,7 @@ string stime[TIMESTRSIZE], sfirstcut[TIMESTRSIZE], slastcut[TIMESTRSIZE];
 				stime
 				);
 	}
+
 	if (cmdarg.numcuts > 0) {
 		firstcut = get_cut_value (0);
 		lastcut = get_cut_value (cmdarg.numcuts - 1);
@@ -492,7 +534,8 @@ string stime[TIMESTRSIZE], sfirstcut[TIMESTRSIZE], slastcut[TIMESTRSIZE];
 		"Swap channels:      %s\n"
 		"Do ->mono->stereo:  %s\n"
 		"Find silences:      %s  %s\n"
-		"Add silence:        %li C, %li C\n"
+		"Add silence:        %li C = %s min, %li C = %s min\n"
+		"Fade-in / -out:     %li C = %s min, %li C = %s min\n"
 		"Input data size:    %s\n"
 		"Number of cuts:     %i\n"
 		"Skipping before:    %s min = %li C  (excl) (first cut)\n"
@@ -510,8 +553,10 @@ string stime[TIMESTRSIZE], sfirstcut[TIMESTRSIZE], slastcut[TIMESTRSIZE];
 		cmdarg.swapwords ? yes : no,
 		cmdarg.monostereo ? yes : no,
 		silence,  cmdarg.silenceinfo ? "[INFO]" : "",
-		cmdarg.startsilence / CDAUDIOSECTORSIZE,
-			cmdarg.endsilence / CDAUDIOSECTORSIZE,
+		cmdarg.startsilence / CDAUDIOSECTORSIZE, stasil,
+			cmdarg.endsilence / CDAUDIOSECTORSIZE, endsil,
+		cmdarg.fadein / CDAUDIOSECTORSIZE, filength, 
+			cmdarg.fadeout / CDAUDIOSECTORSIZE, folength,
 		insize,
 		cmdarg.numcuts,
 		sfirstcut, firstcut,
@@ -528,26 +573,37 @@ string stime[TIMESTRSIZE], sfirstcut[TIMESTRSIZE], slastcut[TIMESTRSIZE];
 /*
 	Generic: get size number, in bytes
 	Converts a number (integer) with an optional unit,
-	units:
-		b (bytes), 
-		c or C (audio CD sectors, default),
-		s (seconds of CD audio)
+	units:      type:
+		b       int   (bytes), 
+		c or C  int   (audio CD sectors, default),
+		s       float (seconds of CD audio)
 	In: string representation of the number
 	Out: ---
-	Return: number IN BYTES
+	Return: number IN BYTES, always a multiple of CDAUDIOSECTORSIZE
 */
 static signed long get_num_unit (const string *s)
 {
 signed long int i;
+float sec;
 char unit[] = "\0";
 int r;
 
 	r = sscanf (s, "%li%1s", &i, unit);
 	if (r == 1)
 		unit[0] = 'C';	/* default unit */
-	if (r < 1  OR  (r == 2  AND  strchr ("bscC", unit[0]) == NULL))
+	if (r < 1  OR  (r == 2  AND  strchr ("bscC.", unit[0]) == NULL))
 		exit_error (ERR_CMDARG, "number/unit syntax: ", s);
-	DBGPRINTF2 ("<< get val: %li %c", i, unit[0]);
+	if (r == 2  AND  unit[0] == '.') {
+		unit[0] = '\0';
+		r = sscanf (s, "%f%1s", &sec, unit);
+		if (r == 2  AND  strchr ("bcC", unit[0]) != NULL)
+			exit_error (ERR_CMDARG,
+			 "number/unit syntax (fractions not with b, c): ", s);
+		if (r < 1  OR  unit[0] != 's')
+			exit_error (ERR_CMDARG, "number/unit syntax: ", s);
+		unit[0] = 'S';
+	}
+	DBGPRINTF3 ("<< get val: %li %2.2f %c", i, sec, unit[0]);
 
 	switch (unit[0]) {
 	case 'c':
@@ -555,10 +611,16 @@ int r;
 		i *= CDAUDIOSECTORSIZE;
 		break;
 	case 'b':	/* bytes */
+		i = ((i + (CDAUDIOSECTORSIZE / 2)) / CDAUDIOSECTORSIZE)
+				* CDAUDIOSECTORSIZE;
 		break;
 	case 's':	/* seconds of CD audio */
 		i *= (signed long) CDSECTORSPERSEC * (signed long) CDAUDIOSECTORSIZE;
 		break;
+	case 'S':
+		i = ((signed long) (sec * (float) CDSECTORSPERSEC + 0.5))
+				* (signed long) CDAUDIOSECTORSIZE;
+		break;
 	default:
 		exit_error (ERR_INTERNAL, "switch", NULL);
 	}		
@@ -613,41 +675,83 @@ signed long l;
 
 
 
+/*
+	In: ---
+	Out: ---
+	Return: Input size in full CD blocks, or -1 if not available
+*/
+signed long get_input_size_in_cd_blocks (void)
+{
+signed long size;
+
+	if (input_size == -1)
+		return -1;
+
+	/* this is file format dependent!!, but input_size allows for that */
+	size = (input_size / CDAUDIOSECTORSIZE);
+	if ((size * CDAUDIOSECTORSIZE) < input_size)
+		size++; /* measure from the true end of the cdr file */
+	return size;
+
+} /* get_input_size_in_cd_blocks() */
+
+
+
+/*
+	Calculates the cut position from the file size for negative cut numbers.
+	Expects the input data size to be initialised.
+*/
+static signed long get_neg_cut (signed long cut)
+{
+signed long pos, size;
+
+	if (input_size == -1)
+		exit_error (ERR_CMDARG, 
+			"cut numbers must not be negative when input can't be sized", 
+			NULL);
+
+	size = get_input_size_in_cd_blocks ();
+	pos = size - (-cut);
+	if (pos < 0) {
+		string buf[100];
+		sprintf (buf, 
+			"cut %li C before start of input with length %li b = %li C",
+			(long) cut, (long) input_size, (long) size);
+		exit_error (ERR_CMDARG, buf, NULL);
+	}
+	return pos;
+} /* get_neg_cut() */
+
+
+
 /*
 	Get the Nth cut value.
 	The value returned is positive and in audio CD sectors!
+	Expects the input data size to be initialised.
 */
 unsigned long get_cut_value (int n)
 {
-signed long cut, l;
+signed long cut, l = 0;
 
-	l = 0;
 	cut = get_num_unit (cmdarg.cutstarts[n]) / CDAUDIOSECTORSIZE;
-	if ((cut == 0L) AND (n == (cmdarg.numcuts - 1)))
-		/* a 0 as last cut number means all the rest */
-		l = ULONG_MAX / CDAUDIOSECTORSIZE;
-	else if (cut < 0) {
+	l = cut;
+	if ((cut == 0L) AND (n == (cmdarg.numcuts - 1))) {
+		/* a 0 as last cut number means all the rest;
+		   if input size is not available it's better to use the theoretical
+		   maximum file size than to fail */
+		if (input_size != -1)
+			l = get_neg_cut (cut);
+		else
+			l = ULONG_MAX / CDAUDIOSECTORSIZE;
+	} else if (cut < 0) {
 		/* negative cut number - calculate from file size */
 		if (input_size == -1)
 			exit_error (ERR_CMDARG, 
 				"cut numbers must not be negative when input can't be sized", 
 				NULL);
-		else {
-			/* get size of file in full cd blocks */
-			l = (input_size / CDAUDIOSECTORSIZE);
-			if ((l * CDAUDIOSECTORSIZE) < input_size)
-				l++; /* measure from the true end of the cdr file */
-			if (-cut > l) {
-				string buf[100];
-				sprintf (buf, 
-					"cut %li C before start of input with length %li b = %li C",
-					(long) cut, (long) input_size, (long) l);
-				exit_error (ERR_CMDARG, buf, NULL);
-			}
-			l = l + cut;
-		}
-	} else
-		l = cut;
+		else
+			l = get_neg_cut (cut);
+	}
 	
 	DBGPRINTF2 ("get_cut_value(%i): %li\n", n, (long) l);
 	return (unsigned long) l;
@@ -695,9 +799,12 @@ void get_cmdarg_info (cmdarg_t *pcmdarg)
 */
 void init_process_info (process_t *pinfo)
 {
+	pinfo->n_in = pinfo->n_returned = 0;
+
+	pinfo->first = TRUE;
+	pinfo->last = FALSE;
 
 	pinfo->little_host = is_localhost_little ();
-
 	pinfo->from_little = cmdarg.little_input;
 	pinfo->to_little = cmdarg.little_output;
 	pinfo->iscale = cmdarg.iscale;
@@ -707,11 +814,10 @@ void init_process_info (process_t *pinfo)
 	pinfo->silencecuts = cmdarg.silencecuts;
 	pinfo->silence_thresh = cmdarg.silence_thresh;
 	pinfo->silence_delay = cmdarg.silence_delay;
-	pinfo->first = TRUE;
-	pinfo->last = FALSE;
-
-	pinfo->n_in = pinfo->n_returned = 0;
-
+	pinfo->silence_val = cmdarg.verbose;
+	pinfo->fadein = 0;
+	pinfo->fadeout = 0;
+	
 } /* init_process_info() */
 
 
