@@ -7,7 +7,7 @@ Max line length:    80
 Programmer:         Volker Kuhlmann
 
 
-wav2cdr 2.1 Copyright (C) 1997, 1998 Volker Kuhlmann
+wav2cdr 2.3.2 Copyright (C) 1997, 1998, 1999 Volker Kuhlmann
 This program is free software under the terms of the GNU General Public License
 version 2 (or later, at your option).
 See the file COPYING for details about license terms and warranty.
@@ -418,10 +418,59 @@ size_t i;
 
 
 
+/*
+*/
+void process_fading (void * buf, process_t *pinfo)
+{
+float fade;
+size_t i;
+static cdseccount_t fi_len = 0, fo_len = 0;
+
+	#ifndef NO_ASSERTMANY
+	assert ((pinfo->n_in BITAND 0x1) == 0);
+	#endif
+
+	/* initialise when fading starts */
+	if (fi_len == 0 AND pinfo->fadein != 0) {
+		fi_len = pinfo->fadein + 1;
+	}
+	if (fo_len == 0 AND pinfo->fadeout != 0) {
+		fo_len = pinfo->fadeout + 1;
+	}
+	DBGPRINTF2 ("fading in %li, out %li\n", fi_len, fo_len);
+	
+	/* fade-in and fade-out, possibly combined */
+	if (fi_len > 0) {
+		if (fo_len > 0) {
+			fade = (((float) (fi_len - pinfo->fadein)) / ((float) fi_len))
+					* (((float) pinfo->fadeout) / ((float) fo_len));
+			if (--pinfo->fadein == 0) fi_len = 0;
+			if (--pinfo->fadeout == 0) fo_len = 0;
+		} else {
+			fade = ((float) (fi_len - pinfo->fadein)) / ((float) fi_len);
+			if (--pinfo->fadein == 0) fi_len = 0;
+		}
+	} else {
+		if (fo_len > 0) {
+			fade = ((float) pinfo->fadeout) / ((float) fo_len);
+			if (--pinfo->fadeout == 0) fo_len = 0;
+		} else {
+			return; /* both fi_len and fo_len are 0, no fading */
+		}
+	}
+	DBGPRINTF1 ("fading scale: %2.5f\n", fade);
+	
+	for (i = 0; i < (pinfo->n_in / 2); i++)
+		(AS_S16(buf)[i]) = (SINT16) (((float) (AS_S16(buf)[i])) * fade);
+
+} /* process_fading() */
+
+
+
 /*
 	Decide whether this sector is silent
 	In: buffer
-	Out: ---
+	Out: buffer content '\0'-terminated
 	Return: TRUE if sector is silent
 */
 static BOOL is_silent_sector (void * buf, const process_t *pinfo)
@@ -442,11 +491,23 @@ static size_t secnum = 0;
 BOOL is_silent;
 
 #define ABS(arg) (((arg) < 0) ? (-(arg)) : (arg))
+#define BUF(buf) ((char *) (buf) + strlen (buf))
 
 	if ((pinfo->n_in / 2) == 0) {
+		* (char *) buf = '\0';
 		return TRUE;	/* nothing to do... */
 	}
 
+	/* fast case for threshold == 0: because the silence value of this sector
+		can not be less than zero, we treat this as "this is a silent sector if
+		all samples are 0" */
+	if (pinfo->silence_thresh == 0) {
+		for (i = 0; i < (pinfo->n_in / 2); i++)
+			if (AS_S16(buf)[i] != 0)
+				return FALSE;
+		return TRUE;
+	}
+
 	/* calculate averages: (sum(x)/N), and (sum(abs(x))/N)
 	*/
 	for (i = 0; i < (pinfo->n_in / 2); i++) {
@@ -502,8 +563,9 @@ BOOL is_silent;
 	is_silent = (avgdcpk < pinfo->silence_thresh);
   #endif
 
+	* (char *) buf = '\0';
 	#ifdef DEBUG
-	sprintf (((char *) buf) + strlen (buf),
+	sprintf (buf,
 		" %li: av %6i, +av %5i, d %3i, avdc %5i"
 		", s%7li, +s %6li, +sdc %5li, n %i, =%c\n",
 		(long) secnum++,
@@ -511,7 +573,11 @@ BOOL is_silent;
 		sum, sumabs, sumdcpk, pinfo->n_in / 2, is_silent ? 'Y' : 'N');
 		/* note silence_info() scans for this = sign */
 	#endif
+	/* verbose output of how silent this sector is */
+	if (pinfo->silence_val) sprintf (BUF(buf), "S%hu ", avgdcpk);
+	
 	return is_silent;	
+#undef BUF
 } /* is_silent_sector() */
 
 
@@ -537,7 +603,7 @@ void process_silencecuts (void * buf, process_t *pinfo)
 {
 static cdseccount_t secnum = 0; 	/* counter of sectors */
 static BOOL sigstarted = FALSE; 	/* a non-silent interval has started */
-static cdseccount_t sigstart;		/* start of signal */
+static cdseccount_t sigstart;		/* start of signal (i.e. non-silence) */
 static cdseccount_t
 			 silentsectors = 0; 	/* silent sectors so far, this interval */
 static cdseccount_t silencestart; 	/* first sector of silence, this interval */
@@ -556,11 +622,11 @@ static cdseccount_t lastsigend = 0;
 	/* in case is_silent_sector() wrote something: */
 	#define BUF ((char *) buf + strlen (buf))
 #else
-	#define BUF buf
+	/*#define BUF buf*/
+	#define BUF ((char *) buf + strlen (buf))
 #endif
 	
-	* (char *) buf = '\0';
-
+	/* is_silent_sector() will put a '\0' into the buffer returned */
 	if (is_silent_sector (buf, pinfo)) {
 		if ((pinfo->n_in / 2) > 0U) {
 			if (silentsectors++ == 0U)
@@ -568,10 +634,10 @@ static cdseccount_t lastsigend = 0;
 			if (silentsectors == pinfo->silence_delay  AND  sigstarted) {
 			#ifdef HARDON_CUT
 				DBGPRINTF2 ("cut: signal: %ldC,%ldC\n", sigstart, silencestart);
-				sprintf (BUF, "%ldC\n%ldC\n", sigstart, silencestart);
+				sprintf (BUF, "%ldC %ldC\n", sigstart, silencestart);
 			#else
 				DBGPRINTF2 ("cut: signal: %ldC,%ldC\n", sigstart, secnum + 1U);
-				sprintf (BUF, "%ldC\n%ldC\n", sigstart, secnum + 1U);
+				sprintf (BUF, "%ldC %ldC\n", sigstart, secnum + 1U);
 				lastsigend = secnum + 1U;
 			#endif
 				sigstarted = FALSE;
@@ -596,7 +662,7 @@ static cdseccount_t lastsigend = 0;
 	
 	if (pinfo->last  AND  sigstarted) {
 		DBGPRINTF2 ("cut: signal: %ldC,%ldC\n", sigstart, secnum);
-		sprintf (BUF, "%ldC\n%ldC\n", sigstart, secnum);
+		sprintf (BUF, "%ldC %ldC\n", sigstart, secnum);
 	}
 	
 	#undef BUF
@@ -616,6 +682,8 @@ static cdseccount_t lastsigend = 0;
 	Byte swapping and scaling is handled here.
 	Runs on big and little endian machines.
 	Handles little and big input data, generates little or big output.
+	Some care was taken to make things more efficient depending on the
+	processing to be done.
 	In: buffer, processing info
 	Out: buffer contents changed
 	Return: ---
@@ -634,18 +702,18 @@ BOOL order;
 	/* do any byte swapping and scaling if necessary */
 	if (pinfo->iscale != 100)
 		process_swap_iscale (buf, pinfo);
-	else if (pinfo->fscale != 1.0)
-		/* is a precise float comparison ok here? */
+	else if (pinfo->fscale != 1.0) /* is a precise float comparison ok here? */
 		process_swap_fscale (buf, pinfo);
 	else
 		process_swap_noscale (buf, pinfo);
-	/* now byte order is that of target! */
+	/* now byte order of data is that of target audio format! */
 	
 	/* More processing with byte swapping to do? */
 	if (pinfo->monostereo != MS_NONE
-		OR pinfo->silencecuts ) {
+		OR pinfo->silencecuts
+		OR pinfo->fadein != 0 OR pinfo->fadeout != 0) {
 
-		/* get local byte order, currently byte order is that of target */
+		/* get byte order of local architecture, swap data to it if necessary */
 		order = pinfo->from_little;
 		pinfo->from_little = pinfo->to_little;
 		process_swap_tolocal (buf, pinfo);
@@ -667,7 +735,12 @@ BOOL order;
 		/* find silent intervals, and output cut numbers accordingly */
 		if (pinfo->silencecuts)
 			process_silencecuts (buf, pinfo);
+		
+		/* process fading */
+		if (pinfo->fadein != 0 OR pinfo->fadeout != 0)
+			process_fading (buf, pinfo);
 
+		/* swap data back to byte order of target audio format, if necessary */
 		process_swap_totarget (buf, pinfo);
 		pinfo->from_little = order;
 	}
